/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.io.*;
import javax.swing.*;

public class CharCheckThread extends Thread
{
   public CharCheckThread(MakeJmlrBookGUI application, File file)
   {
      super();
      app = application;
      this.file = file;
      init();
   }

   public CharCheckThread(MakeJmlrBookGUI application, BookRoot root)
   {
      super();
      app = application;
      this.root = root;
      init();
   }

   private void init()
   {
      frame = new JFrame(app.getLabel("charcheckresults.title"));

      editorPane = new JEditorPane();
      editorPane.setContentType("text/html");
      editorPane.setEditable(false);
      editorPane.addHyperlinkListener(app);

      frame.getContentPane().add(new JScrollPane(editorPane), "Center");
   }

   public void run()
   {
      infoContent = new StringBuilder(1024);
      infoContent.append( 
        "<h1>"+app.getLabel("charcheckresults.title")+"</h1>");

      app.setWaitCursor();

      if (root != null)
      {
         root.walk(new BookWalkListener()
         {
            public FileElement walkAction(FileElement element)
            {
               if (!element.isDirectory() && element.isNodeSelected())
               {
                  try
                  {
                     checkFile(element.getFile());
                  }
                  catch (IOException e)
                  {
                     infoContent.append(e.getMessage());

                     if (app.inDebugMode())
                     {
                        app.printAndLogMessage(e.getMessage());
                        e.printStackTrace();
                     }
                  }
               }

               return null;
            }
         });
      }
      else
      {
         try
         {
            checkFile(file);
         }
         catch (IOException e)
         {
            infoContent.append(e.getMessage());

            if (app.inDebugMode())
            {
               app.printAndLogMessage(e.getMessage());
               e.printStackTrace();
            }
         }
      }

      editorPane.setText("<style>.error {font-weight: bold; color: red;}</style>"
        +new String(infoContent));

      infoContent = null;

      frame.pack();
      frame.setLocationRelativeTo(app);

      app.setDefaultCursor();

      frame.setVisible(true);
      app.setInfo(app.getLabel("message.done"));

      app.workerThreadFinished(this);
   }

   private void checkFile(File f)
     throws IOException
   {
      MakeJmlrBookGUI.debug("Starting char check on "+f);
      infoContent.append("<h2>"+f.getAbsolutePath()+"</h2>");

      BufferedReader in = null;
      boolean foundInFile = false;

      try
      {
         app.setInfo(app.getLabelWithValue("message.checking",
           f.getAbsolutePath()));
         in = new BufferedReader(new FileReader(f));

         int lineNum = 0;

         String line;

         StringBuilder buffer = new StringBuilder(1024);

         while ((line = in.readLine()) != null)
         {
            lineNum++;

            int n = line.length();

            buffer.setLength(0);

            boolean found = false;

            for (int i = 0; i < n; i++)
            {
               char c = line.charAt(i);

               if ((c >= 0x20 && c <= 0x7E) || c == '\t')
               {
                  if (c == '&')
                  {
                     buffer.append("&amp;");
                  }
                  else if (c == '<')
                  {
                     buffer.append("&lt;");
                  }
                  else if (c == '>')
                  {
                     buffer.append("&gt;");
                  }
                  else if (c == '$')
                  {
                     buffer.append("&#x0024;");
                  }
                  else
                  {
                     buffer.append(c);
                  }
               }
               else
               {
                  if (c == 0xFEFF)
                  {
                     buffer.append("<font class=\"error\">&lt;feff&gt;</font>");
                  }
                  else
                  {
                     buffer.append("<font class=\"error\">"+c+"</font>");
                  }
                  found = true;
                  foundInFile = true;
               }
            }

            if (found)
            {
               infoContent.append(app.getLabelWithValues("charcheckresults.line",
                 new String[]{""+lineNum, new String(buffer)}));
            }
         }
      }
      finally
      {
         if (foundInFile)
         {
            infoContent.append(String.format("<p><a href=\"%s\">%s</a>",
              f.toURI().toString(), app.getLabel("main.edit")));
         }
         else
         {
            infoContent.append(app.getLabel("charcheckresults.nonefound"));
         }

         if (in != null)
         {
            in.close();
         }
      }
   }

   private MakeJmlrBookGUI app;

   private BookRoot root;
   private File file;
   private StringBuilder infoContent;

   private JEditorPane editorPane;
   private JFrame frame;
}
