/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.awt.Cursor;
import java.util.Vector;
import java.io.File;

import java.awt.Dimension;
import javax.swing.*;

public class ClearAuxThread extends Thread
{
   public ClearAuxThread(MakeJmlrBookGUI application, Book book)
   {
      super();

      app = application;
      this.book = book;
   }

   public void run()
   {
      app.setWaitCursor();
      app.disableTools();

      Vector<File> auxList = new Vector<File>();

      book.addAuxFiles(auxList);

      if (auxList.size() == 0)
      {
         JOptionPane.showMessageDialog(app,
            app.getLabel("message.no_aux_files"),
            app.getLabel("message.title"),
            JOptionPane.INFORMATION_MESSAGE);

         return;
      }

      FileListPanel panel = new FileListPanel(auxList);

      JScrollPane sp = new JScrollPane(panel);

      Dimension appDim = app.getSize();
      Dimension spDim = sp.getPreferredSize();

      appDim.height = 3*appDim.height/4;
      appDim.width = 3*appDim.width/4;

      if (spDim.height > appDim.height)
      {
         spDim.height = appDim.height;
      }

      if (spDim.width > appDim.width)
      {
         spDim.width = appDim.width;
      }

      sp.setPreferredSize(spDim);

      if (JOptionPane.showConfirmDialog(app, sp,
            app.getLabel("message.confirm_rm"),
            JOptionPane.OK_CANCEL_OPTION,
            JOptionPane.QUESTION_MESSAGE) != JOptionPane.OK_OPTION)
      {
         app.setInfo(app.getLabel("message.cancelled"));
         return;
      }

      try
      {
         for (int i = 0, n = panel.fileCount(); i < n; i++)
         {
            File file = panel.getFileIfSelected(i);

            if (file != null && file.exists())
            {
               app.setInfoAndDebug(app.getLabelWithValue("message.deleting",
                  file.getAbsolutePath()));

               file.delete();
            }
         }
      }
      catch (Exception e)
      {
         app.error(e.getClass().toString()+"\n"+e.getMessage());
         e.printStackTrace();
      }

      app.enableTools();
      app.setInfo(app.getLabel("message.done"));
      app.setDefaultCursor();
   }

   private Book book;
   private MakeJmlrBookGUI app;
} 

class FileListPanel extends JPanel
{
   public FileListPanel(Vector<File> auxList)
   {
      super(null);
      setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

      setAlignmentX(JComponent.LEFT_ALIGNMENT);

      for (File file : auxList)
      {
         JCheckBox checkBox = new JCheckBox(file.getAbsolutePath(), true);
         checkBox.setAlignmentX(JComponent.LEFT_ALIGNMENT);
         add(checkBox);
      }
   }

   public int fileCount()
   {
      return getComponentCount();
   }

   public File getFile(int index)
   {
      JCheckBox checkBox = (JCheckBox)getComponent(index);

      return new File(checkBox.getText());
   }

   public File getFileIfSelected(int index)
   {
      JCheckBox checkBox = (JCheckBox)getComponent(index);

      if (checkBox.isSelected())
      {
         return new File(checkBox.getText());
      }

      return null;
   }
}
