/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.awt.*;
import java.io.*;
import java.net.*;
import java.awt.event.*;
import javax.swing.*;

public class JMLRResources
{
    public static void error(Component parent, String message)
    {
       MakeJmlrBookGUI.debug("Error: '"+message+"'");

       errorPanel.updateMessage(message);

       JOptionPane.showMessageDialog(parent, errorPanel,
          MakeJmlrBookGUI.getLabelWithAlt("error.title", "Error"),
          JOptionPane.ERROR_MESSAGE);
    }

    public static synchronized void error(Component parent, Exception e)
    {
       String message = e.getMessage();

       if (message == null)
       {
          message = e.getClass().getName();
       }

       Throwable cause = e.getCause();

       if (cause != null)
       {
          message += "\n"+cause.getMessage();
       }

       error(parent, message, e);
    }

    public static void error(Component parent, String message, Exception e)
    {
       MakeJmlrBookGUI.debug(e);

       errorPanel.updateMessage(message, e);

       JOptionPane.showMessageDialog(parent, errorPanel,
          MakeJmlrBookGUI.getLabelWithAlt("error.title", "Error"),
         JOptionPane.ERROR_MESSAGE);
    }


    public static void warning(Component parent, String message)
    {
       MakeJmlrBookGUI.debug("Warning: '"+message+"'");

       errorPanel.updateMessage(message);

       JOptionPane.showMessageDialog(parent,
          errorPanel,
          MakeJmlrBookGUI.getLabelWithAlt("error.warning", "Warning"),
          JOptionPane.WARNING_MESSAGE);
    }

    public static JButton createContinueButton(ActionListener listener)
    {
       return createContinueButton(listener, null);
    }

    public static JButton createContinueButton(ActionListener listener, String tooltipText)
    {
       return createActionButton(
          MakeJmlrBookGUI.getLabel("button.continue"),
          MakeJmlrBookGUI.getMnemonic("button.continue"),
          "okay", listener,
          KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), tooltipText);
    }

    public static JButton createOkayButton(ActionListener listener)
    {
       return createOkayButton(listener, null);
    }

    public static JButton createOkayButton(ActionListener listener, String tooltipText)
    {
       return createActionButton(
          MakeJmlrBookGUI.getLabel("button.okay"),
          MakeJmlrBookGUI.getMnemonic("button.okay"),
          "okay", listener,
          KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), tooltipText);
    }

    public static JButton createCancelButton(ActionListener listener)
    {
       return createCancelButton(listener, null);
    }

    public static JButton createCancelButton(ActionListener listener, String tooltipText)
    {
       return createActionButton(
          MakeJmlrBookGUI.getLabel("button.cancel"),
          MakeJmlrBookGUI.getMnemonic("button.cancel"),
          "cancel", listener,
          KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), tooltipText);
    }

    public static JButton createCloseButton(ActionListener listener)
    {
       return createCloseButton(listener, null);
    }

    public static JButton createCloseButton(ActionListener listener, String tooltipText)
    {
       return createActionButton(
          MakeJmlrBookGUI.getLabel("button.close"),
          MakeJmlrBookGUI.getMnemonic("button.close"),
          "close", listener,
          KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), tooltipText);
    }

    public static JButton createActionButton(String parent,
      String tag, ActionListener listener, KeyStroke keyStroke)
    {
       String label = parent+"."+tag;

       return createActionButton(MakeJmlrBookGUI.getLabel(label),
         MakeJmlrBookGUI.getMnemonic(label),
         tag, listener, keyStroke,
         MakeJmlrBookGUI.getLabelWithAlt(label+".tooltip", null));
    }

    public static JButton createActionButton(String label, char mnemonic,
      String actionCommand, ActionListener listener, KeyStroke keyStroke, 
      String tooltipText)
    {
       JButton button = new JButton(label);
       button.setMnemonic(mnemonic);

       if (listener != null)
       {
          button.addActionListener(listener);

          if (actionCommand != null)
          {
             button.setActionCommand(actionCommand);

             if (keyStroke != null)
             {
                button.registerKeyboardAction(listener, actionCommand, keyStroke,
                  JComponent.WHEN_IN_FOCUSED_WINDOW);
             }
          }
       }

       if (tooltipText != null)
       {
          button.setToolTipText(tooltipText);
       }

       return button;
    }

    public static JButton createImageButton(String parent, 
      String tag, String imgRef, ActionListener listener)
    {
       return createImageButton(parent, tag, imgRef, listener, null);
    }

    public static JButton createImageButton(String parent, 
      String tag, String imgRef, ActionListener listener, 
      KeyStroke keyStroke)
    {
       return createImageButton(parent, tag, imgRef, listener,
          keyStroke, 
          MakeJmlrBookGUI.getLabelWithAlt(parent+"."+tag+".tooltip", null));
    }

    public static JButton createImageButton(String parent, 
     String tag, String imgRef, ActionListener listener, 
     KeyStroke keyStroke, String tooltip)
    {
       JButton button = new JButton();

       String label = parent+"."+tag;

       String text = MakeJmlrBookGUI.getLabel(label);

       String location = imgRef.endsWith(".gif") ?
          "/toolbarButtonGraphics/"+imgRef :
          "/icons/"+imgRef;

       URL imageURL = MakeJmlrBookGUI.class.getResource(location);

       if (imageURL == null)
       {
          button.setText(text);
          System.err.println("Can't find image resource '"+
            location+"'");
       }
       else
       {
           ImageIcon ic = new ImageIcon(imageURL, text);

           button.setIcon(ic);

           Dimension dim = new Dimension(ic.getIconWidth()+2,
             ic.getIconHeight()+2);

           button.setPreferredSize(dim);
           button.setMinimumSize(dim);
           button.setSize(dim);
           button.setMaximumSize(dim);

           if (tooltip == null)
           {
              tooltip = text;
           }
       }

       if (listener != null)
       {
          button.addActionListener(listener);

          button.setActionCommand(tag);

          if (keyStroke != null)
          {
             button.registerKeyboardAction(listener, tag, keyStroke,
               JComponent.WHEN_IN_FOCUSED_WINDOW);
          }
       }

       if (tooltip != null)
       {
          button.setToolTipText(tooltip);
       }

       return button;
    }

    public static JButton createUpButton(ActionListener listener,
      KeyStroke keyStroke, String tooltipText)
    {
       return createImageButton("button", "up", "navigation/Up24.gif",
         listener, keyStroke, tooltipText);
    }

    public static JButton createUpButton(ActionListener listener)
    {
       return createImageButton("button", "up", "navigation/Up24.gif",
         listener);
    }

    public static JButton createDownButton(ActionListener listener,
      KeyStroke keyStroke, String tooltipText)
    {
       return createImageButton("button", "down", "navigation/Down24.gif", listener, keyStroke, tooltipText);
    }

    public static JButton createDownButton(ActionListener listener)
    {
       return createImageButton("button", "down", "navigation/Down24.gif", listener);
    }

    public static JButton createAddButton(ActionListener listener,
      KeyStroke keyStroke, String tooltipText)
    {
       return createImageButton("button", "add", "general/Add24.gif",
        listener, keyStroke, tooltipText);
    }

    public static JButton createAddButton(ActionListener listener)
    {
       return createImageButton("button", "add", "general/Add24.gif",
        listener);
    }

    public static JButton createRemoveButton(ActionListener listener,
      KeyStroke keyStroke, String tooltipText)
    {
       return createImageButton("button", "remove", "general/Delete24.gif", listener, keyStroke, tooltipText);
    }

    public static JButton createRemoveButton(ActionListener listener)
    {
       return createImageButton("button", "remove", "general/Delete24.gif", listener);
    }

    public static JRadioButton createRadioButton(String parentTag,
      String tag, ActionListener listener, boolean selected)
    {
       String label = parentTag+"."+tag;

       JRadioButton button = new JRadioButton(
          MakeJmlrBookGUI.getLabel(label), selected);

       button.setMnemonic(MakeJmlrBookGUI.getMnemonic(label));
       button.setActionCommand(tag);

       if (listener != null)
       {
          button.addActionListener(listener);
       }

       String toolTip = MakeJmlrBookGUI.getLabelWithAlt(label+".tooltip", null);

       if (toolTip != null)
       {
          button.setToolTipText(toolTip);
       }

       return button;
    }

   public static JTextArea createMessageArea()
   {
      return createMessageArea(8, 30);
   }

   public static JTextArea createMessageArea(String label)
   {
      return createMessageArea(8, 30, label);
   }

   public static JTextArea createMessageArea(int rows, int cols, String label)
   {
      JTextArea area = createMessageArea(rows, cols);

      area.setText(MakeJmlrBookGUI.getLabel(label));

      return area;
   }

   public static JTextArea createMessageArea(int rows, int cols)
   {
      JTextArea area = new JTextArea(rows, cols);
      area.setWrapStyleWord(true);
      area.setLineWrap(true);
      area.setEditable(false);
      area.setBorder(null);
      area.setOpaque(false);

      return area;
   }


    private static ErrorPanel errorPanel = new ErrorPanel();
}
