/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.io.File;
import java.io.IOException;
import java.io.EOFException;
import java.nio.charset.Charset;

import com.dickimawbooks.texparserlib.*;
import com.dickimawbooks.texparserlib.primitives.*;
import com.dickimawbooks.texparserlib.generic.*;
import com.dickimawbooks.texparserlib.latex.*;
import com.dickimawbooks.texparserlib.latex2latex.*;

// Check for "pseudo" package. That is, a .sty file
// that could simply be input using \\input instead of \\usepackage.
// Any use of an internal command, will reject this as a pseudo
// package. Also, package commands such as \NeedsTeXFormat,
// \DeclarePackage, \ProvidesOption, \ExecuteOption
// or \RequirePackage will produce a false value.
public class PseudoPackageParser extends DefaultTeXParserListener
  implements Writeable
{
   public PseudoPackageParser(TeXApp texApp)
    throws IOException
   {
      super(null);
      this.texApp = texApp;
      setWriteable(this);
   }

   public void skipping(Ignoreable ignoreable)
      throws IOException
   {
   }

   public TeXApp getTeXApp()
   {
      return texApp;
   }

   protected void addPredefined()
   {
      putControlSequence(new EndInput());
   }

   public ControlSequence createUndefinedCs(String name)
   {
      return new PPPUndefined(name);
   }

   public Writeable getWriteable()
   {
      return this;
   }

   public void write(String text)
     throws IOException
   {
   }

   public void writeln(String text)
     throws IOException
   {
   }

   public void write(char c)
     throws IOException
   {
   }

   public void writeCodePoint(int codePoint)
     throws IOException
   {
   }

   public void overwithdelims(TeXObject firstDelim,
     TeXObject secondDelim, TeXObject before, TeXObject after)
    throws IOException
   {
   }

   public void abovewithdelims(TeXObject firstDelim,
     TeXObject secondDelim, TeXDimension thickness, TeXObject before,
     TeXObject after)
    throws IOException
   {
   }

   public void substituting(String original, String replacement)
   {
   }

   public void href(String url, TeXObject text)
      throws IOException
   {
   }

   public void subscript(TeXObject arg)
     throws IOException
   {
   }

   public void superscript(TeXObject arg)
     throws IOException
   {
   }

   public static boolean isPseudoPackage(TeXApp app, File file)
     throws IOException
   {
      PseudoPackageParser listener = new PseudoPackageParser(app);
      listener.setIsPseudo(true);

      try
      {
         TeXParser parser = new TeXParser(listener);

         parser.setCatCode('@', TeXParser.TYPE_LETTER);
         parser.parse(file);
      }
      catch (IOException e)
      {
         listener.setIsPseudo(false);
      }

      return listener.getIsPseudoState();
   }

   protected void setIsPseudo(boolean state)
   {
      isPseudo = state;
   }

   public boolean getIsPseudoState()
   {
      return isPseudo;
   }

   public Charset getCharSet()
   {
      return null;
   }

   public void endParse(File file)
      throws IOException
   {
      this.file = null;
   }

   public void beginParse(File file)
      throws IOException
   {
      this.file = file;
   }

   public File getFile()
   {
      return file;
   }

   public float exToPt(float value)
   {// not implemented
      return value;
   }

   public float emToPt(float value)
   {// not implemented
      return value;
   }

   private boolean isPseudo;

   private TeXApp texApp;

   private File file = null;
}

class PPPUndefined extends ControlSequence
{
   public PPPUndefined(String name)
   {
      super(name);
   }

   public Object clone()
   {
      return new PPPUndefined(name);
   }

   public void process(TeXParser parser) throws IOException
   {
      if (name.contains("@")
       || name.equals("NeedsTeXFormat")
       || name.equals("\\ProvidesPackage")
       || name.equals("\\DeclareOption")
       || name.equals("\\ExecuteOptions")
       || name.equals("\\RequirePackage"))
      {
         ((PseudoPackageParser)parser.getListener()).setIsPseudo(false);
         throw new EOFException();
      }
   }

   public void process(TeXParser parser, TeXObjectList stack)
    throws IOException
   {
      process(parser);
   }
}
