/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;

import javax.swing.*;

import com.dickimawbooks.texparserlib.*;
import com.dickimawbooks.texparserlib.bib.*;

import com.dickimawbooks.makejmlrbookgui.*;

public class ImportPapersPanel extends JPanel
{
   public ImportPapersPanel(NewBookDialog dialog, 
      ArticlesPanel articlesPanel,
      String name)
   {
      super(null);
      this.dialog = dialog;
      this.articlesPanel = articlesPanel;
      setName(name);

      setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

      addRow();
   }

   public void addRow()
   {
      add(new ImportPaperPanel(dialog, articlesPanel));
      updateButtons();
   }

   public void addRow(File file)
   {
      ImportPaperPanel impPanel = new ImportPaperPanel(dialog, articlesPanel);
      add(impPanel);
      impPanel.setFile(file);
      updateButtons();
   }

   public void addRow(int index)
   {
      add(new ImportPaperPanel(dialog, articlesPanel), index);
      updateButtons();
   }

   public void addRowAfter(Component comp)
   {
      int index = getComponentIndex(comp)+1;

      if (index >= getComponentCount())
      {
         addRow();
      }
      else
      {
         addRow(index);
      }
   }

   public void updatePanel()
   {
      for (int i = 0, n = getComponentCount(); i < n; i++)
      {
         ImportPaperPanel panel = (ImportPaperPanel)getComponent(i);
         panel.updatePanel();
      }
   }

   public int getComponentIndex(Component comp)
   {
      for (int i = 0, n = getComponentCount(); i < n; i++)
      {
         if (getComponent(i) == comp) return i;
      }

      return -1;
   }

   public ImportPaperPanel getImportPaper(File file)
   {
      for (int i = 0, n = getComponentCount(); i < n; i++)
      {
         ImportPaperPanel panel = (ImportPaperPanel)getComponent(i);

         if (panel.hasFile(file))
         {
            return panel;
         }
      }

      return null;
   }

   public int getImportPaperIndex(File file)
   {
      for (int i = 0, n = getComponentCount(); i < n; i++)
      {
         ImportPaperPanel panel = (ImportPaperPanel)getComponent(i);

         if (panel.hasFile(file))
         {
            return i;
         }
      }

      return -1;
   }

   public int getImportPaperIndex(ImportPaperPanel p)
   {
      for (int i = 0, n = getComponentCount(); i < n; i++)
      {
         if ((ImportPaperPanel)getComponent(i) == p)
         {
            return i;
         }
      }

      return -1;
   }

   public void moveRowUp(int index)
   {
      if (index <= 0) return;

      Component comp = getComponent(index);

      remove(comp);
      add(comp, index-1);

      updateButtons();
   }

   public void moveRowDown(int index)
   {
      int n = getComponentCount();

      if (index >= n-1) return;

      Component comp = getComponent(index);
      add(comp, index+1);

      updateButtons();
   }

   public void updateButtons()
   {
      for (int i = 0, n = getComponentCount()-1; i <= n; i++)
      {
         ImportPaperPanel panel = (ImportPaperPanel)getComponent(i);

         panel.enableUpButton(i > 0);
         panel.enableDownButton(i < n);
         panel.enableRemoveButton(n > 0);
      }

      revalidate();
   }

   public int countPapers()
   {
      int total = 0;

      for (int i = 0, n = getComponentCount()-1; i <= n; i++)
      {
         ImportPaperPanel panel = (ImportPaperPanel)getComponent(i);

         if (panel.hasFile()) total++;
      }

      return total;
   }

   public Vector<ImportPaperPanel> getPapers()
   {
      int n = getComponentCount();

      Vector<ImportPaperPanel> list = new Vector<ImportPaperPanel>(n);

      for (int i = 0; i < n; i++)
      {
         ImportPaperPanel panel = (ImportPaperPanel)getComponent(i);

         if (panel.hasFile())
         {
            list.add(panel);
         }
      }

      return list;
   }

   public void addCimlImportFile(File base, BibEntry entry, TeXParser parser)
   {
      ImportPaperPanel lastEmptyComp = null;

      for (int i = 0, n = getComponentCount()-1; i <= n; i++)
      {
         ImportPaperPanel panel = (ImportPaperPanel)getComponent(i);

         if (!panel.hasFile())
         {
            lastEmptyComp = panel;
         }
      }

      if (lastEmptyComp == null)
      {
         lastEmptyComp = new ImportPaperPanel(dialog, articlesPanel);
         add(lastEmptyComp);
      }

      lastEmptyComp.setCimlImportFile(base, entry, parser);

      updateButtons();
   }

   private NewBookDialog dialog;
   private ArticlesPanel articlesPanel;
}
