/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.util.Vector;
import java.io.File;
import java.io.IOException;

import com.dickimawbooks.texparserlib.*;
import com.dickimawbooks.texparserlib.latex.*;
import com.dickimawbooks.texparserlib.latex2latex.*;
import com.dickimawbooks.texparserlib.bib.Contributor;

import com.dickimawbooks.makejmlrbookgui.*;

public class PrefaceParser extends NewBookParser
{
   public PrefaceParser(TeXApp texApp, File outDir,
    Vector<Contributor> editors, String prefTag)
    throws IOException
   {
      super(texApp, outDir);

      this.editors = editors;
      this.prefaceTag = prefTag;
   }

   protected void addPredefined()
   {
      super.addPredefined();

      putControlSequence(new PrefChapter(this));
      putControlSequence(new SignOff("authorsignoff"));
   }

   public boolean isReplaceCmd(String name)
   {
      if (name.equals("chapter"))
      {
         return true;
      }

      return super.isReplaceCmd(name);
   }

   protected void setAppendEndPreface(boolean flag)
   {
      appendEndPreface = true;
   }

   public void endParse(File file)
    throws IOException
   {
      if (appendEndPreface)
      {
         if (doSignOff)
         {
            writeln("");
            writeln("\\begin{signoff}{Insert Date Here}");

            for (Contributor editor : editors)
            {
               writeln("% Editor details: ");
               writeln("\\Editor{" + editor.getForenames()
                +" " +editor.getSurname() +"\\\\");
               writeln("Insert Address Here\\\\");
               writeln("\\mailto{insert@email.here}}");
            }

            writeln("\\end{signoff}");
         }

         writeln("\\end{preface}");
      }

      super.endParse(file);
   }

   public String getPrefaceTag()
   {
      return prefaceTag;
   }

   private boolean appendEndPreface = false, doSignOff=true;

   private Vector<Contributor> editors;

   private String prefaceTag;
}

class PrefChapter extends L2LControlSequence
{
   public PrefChapter(PrefaceParser prefParser)
   {
      super("chapter");
      this.prefParser = prefParser;
   }

   public Object clone()
   {
      return new PrefChapter(prefParser);
   }

   public void process(TeXParser parser, TeXObjectList stack)
     throws IOException
   {
      StringBuffer original = new StringBuffer();

      original.append(toString(parser));

      TeXObject obj = stack.pop();

      if (obj instanceof CharObject 
       && obj.toString().equals("*"))
      {
         original.append('*');
         obj = stack.pop();
         original.append(obj.toString(parser));
      }
      else
      {
         stack.push(obj);

         obj = stack.popArg(parser, '[', ']');

         if (obj != null)
         {
            original.append("["+obj.toString(parser)+"]");
         }

         obj = stack.pop();

         original.append(obj.toString(parser));
      }

      String replacement = "\\begin{preface}";

      String tag = prefParser.getPrefaceTag();

      if (tag != null)
      {
         replacement += "["+tag+"]";
      }

      prefParser.substituting(original.toString(), replacement);

      prefParser.write(replacement);
      prefParser.setAppendEndPreface(true);
   }

   public void process(TeXParser parser)
     throws IOException
   {
      StringBuffer original = new StringBuffer();

      original.append(toString(parser));

      TeXObject obj = parser.popStack();

      if (obj instanceof CharObject 
       && obj.toString().equals("*"))
      {
         original.append('*');
         obj = parser.popStack();
         original.append(obj.toString(parser));
      }
      else
      {
         parser.push(obj);

         obj = parser.popNextArg('[', ']');

         if (obj != null)
         {
            original.append("["+obj.toString(parser)+"]");
         }

         obj = parser.popStack();

         original.append(obj.toString(parser));
      }

      String replacement = "\\begin{preface}";

      String tag = prefParser.getPrefaceTag();

      if (tag != null)
      {
         replacement += "["+tag+"]";
      }

      prefParser.substituting(original.toString(), replacement);

      prefParser.write(replacement);
      prefParser.setAppendEndPreface(true);
   }

   private PrefaceParser prefParser;
}
